/**
 *  @file       AnclkDisplay.cpp
 *  Project     Anclk display library
 *  @brief      An Arduino library for custom made JSC electronics ANCLK display.
 *  @author     Ales Svoboda
 *  License     Apache-2.0 - Copyright (c) 2022 – 2023 JSC electronics
 *  @version    v1.0
 *
 *  Copyright (c) 2022 – 2023 JSC electronics
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#include "AnclkDisplay.h"

AnclkDisplay::AnclkDisplay(uint8_t expAdr1, uint8_t expAdr2, uint8_t expAdr3)
{
    expanderIC7 = new I2cDiscreteIoExpander(expAdr1); 
    expanderIC2 = new I2cDiscreteIoExpander(expAdr2);
    expanderIC4 = new I2cDiscreteIoExpander(expAdr3);  

    expanderIC7->disableBitwiseInversion();
    expanderIC2->disableBitwiseInversion();
    expanderIC4->disableBitwiseInversion();

    Serial.print("I2C expander 4 address: ");Serial.println(expanderIC4->getAddress());
    Serial.print("I2C expander 2 address: ");Serial.println(expanderIC2->getAddress());
    Serial.print("I2C expander 7 address: ");Serial.println(expanderIC7->getAddress());
}

void AnclkDisplay::begin(){
    Wire.begin();
    displayClear();
}

void AnclkDisplay::displayColon(bool state){
    bitWrite(expanderIC2Port, COLON_LED1_PIN, state);
    bitWrite(expanderIC2Port, COLON_LED2_PIN, state);
    expanderIC2->digitalWrite(expanderIC2Port);
}

void AnclkDisplay::displayDot(uint8_t idx, bool state){
    if(isCorrectIdx(idx)){
        switch (idx)
        {
        case 1:
            bitWrite(expanderIC4Port, DISP1_DP_PIN, state);
            expanderIC4->digitalWrite(expanderIC4Port);
            break;
        case 2:
            bitWrite(expanderIC2Port, DISP2_DP_PIN, state);
            expanderIC2->digitalWrite(expanderIC2Port);
            break;
        case 3:
            bitWrite(expanderIC7Port, DISP3_DP_PIN, state);
            expanderIC7->digitalWrite(expanderIC7Port);
            break;    
        case 4:
            bitWrite(expanderIC7Port, DISP4_DP_PIN, state);
            expanderIC7->digitalWrite(expanderIC7Port);
            break;    
        default:
            break;
        }
    }
}

void AnclkDisplay::displayClear(){
    expanderIC7Port = INIT_VAL_PCF8575;
    expanderIC7->digitalWrite(expanderIC7Port);
    expanderIC4Port = INIT_VAL_PCF8575;
    expanderIC4->digitalWrite(expanderIC4Port);
    expanderIC2Port = INIT_VAL_PCF8575;
    expanderIC2->digitalWrite(expanderIC2Port);
}

bool AnclkDisplay::isCorrectIdx(uint8_t idx){
    if(idx > 0 && idx <= NUM_OF_INDEXES){
        return true;
    }
    return false;
}

uint8_t AnclkDisplay::readAllExpanders(uint16_t &exp1, uint16_t &exp2, uint16_t &exp3){
    uint8_t status = expanderIC4->digitalRead();
    if (TWI_SUCCESS == status)
    {
        exp1 = expanderIC4->getPorts();
        // Serial.println(exp1, BIN);
    }
    else
    {
        // display error information on serial console
        Serial.print("IC_4 ERROR: ");
        Serial.print(status, DEC);
        Serial.println(".");
        return status;
    }

    status = expanderIC2->digitalRead();
    if (TWI_SUCCESS == status)
    {
        exp2 = expanderIC2->getPorts();
        // Serial.println(exp2, BIN);
    }
    else
    {
        // display error information on serial console
        Serial.print("IC_2 ERROR: ");
        Serial.print(status, DEC);
        Serial.println(".");
        return status;
    }
    status = expanderIC7->digitalRead();
    if (TWI_SUCCESS == status)
    {
        exp3 = expanderIC7->getPorts();
        // Serial.println(exp3, BIN);
    }
    else
    {
        // display error information on serial console
        Serial.print("IC_7 ERROR: ");
        Serial.print(status, DEC);
        Serial.println(".");
        return status;

    }

    return 0;
}

void AnclkDisplay::printPortsState(){
    Serial.print("IC_4 port BIN: ");
    Serial.print(expanderIC4Port, BIN);
    Serial.println();

    Serial.print("IC_2 port BIN: ");
    Serial.print(expanderIC2Port, BIN);
    Serial.println();

    Serial.print("IC_7 port BIN: ");
    Serial.print(expanderIC7Port, BIN);
    Serial.println();
}

void AnclkDisplay::displaySymbol(byte symbol, DisplayEnum disp) {
uint16_t expPort1, expPort2, expPort3;
  // Read current states from expanders
  /* Attention: for some reason the reading of the expanders status does not work. */
//   if (0 != readAllExpanders(expPort1, expPort2, expPort3)) {
//     return;
//   }
  //printPortsState();
  //printPortsState(expanderIC4Port, expanderIC2Port, expanderIC7Port);
  
  for (int segment = 0; segment < 8; segment++) {
    bool on = symbol & (1 << (7 - segment));
    int pin;

    switch (disp) {
    case DisplayEnum::DISP1: // First digit controlled by IC4
        pin = segmentArray1[segment]; // adjust offset for position  
        on ? bitWrite(expanderIC4Port, pin, 1) : bitWrite(expanderIC4Port, pin, 0);
        // if (on) expanderIC4Port |= (1 << pin);
        // else expanderIC4Port &= ~(1 << pin);
        break;

    case DisplayEnum::DISP2: // Second digit controlled by IC4 and IC2
        pin = segmentArray2[segment]; // adjust offset for position    
        if (segment < 2) {
            on ? bitWrite(expanderIC2Port, pin, 1) : bitWrite(expanderIC2Port, pin, 0);
        } else {
            on ? bitWrite(expanderIC4Port, pin, 1) : bitWrite(expanderIC4Port, pin, 0);
        }
        break;

    case DisplayEnum::DISP3: // Third digit controlled by IC7 and IC2
        pin = segmentArray3[segment]; // adjust offset for position
        if ((segment < 2) || (segment > 4)) {
            on ? bitWrite(expanderIC7Port, pin, 1) : bitWrite(expanderIC7Port, pin, 0);
        } else {
            on ? bitWrite(expanderIC2Port, pin, 1) : bitWrite(expanderIC2Port, pin, 0);
        }
        break;

    case DisplayEnum::DISP4: // Fourth digit controlled by IC7
        pin = segmentArray4[segment]; // adjust offset for position
        on ? bitWrite(expanderIC7Port, pin, 1) : bitWrite(expanderIC7Port, pin, 0);
        break;   
    }
  }

  // Write the port data to the appropriate expanders
  expanderIC2->digitalWrite(expanderIC2Port);
  expanderIC4->digitalWrite(expanderIC4Port);
  expanderIC7->digitalWrite(expanderIC7Port);
  //printPortsState();
  //printPortsState(expanderIC4Port, expanderIC2Port, expanderIC7Port);
}

void AnclkDisplay::printPortsState(uint16_t expanderIC4Port, uint16_t expanderIC2Port, uint16_t expanderIC7Port) {
    // Funkce pro zobrazení jednotlivých displejů
    Serial.println("Displaying states of all four 7-segment displays:");
    
    // Funkce pro zobrazení jednotlivých segmentů pro každý displej
    printSingleDisplay(expanderIC4Port, 5, 6, 9, 0, 1, 3, 2, 4);
    printSingleDisplay((expanderIC2Port & 0b00011000) | (expanderIC4Port >> 7 & 0b11111110), 3, 2, 0, 1, 2, 3, 4, 3);
    printSingleDisplay((expanderIC2Port & 0b00000111) | (expanderIC7Port & 0b11111000), 2, 3, 0, 1, 2, 4, 5, 6);
    printSingleDisplay(expanderIC7Port, 2, 3, 4, 5, 6, 0, 15, 1);
    Serial.println();
}

void AnclkDisplay::showNumber(uint16_t num) {
    if (num > 9999) return;

    uint8_t firstDigit  = num / 1000; // Získá první číslici (tisíce)
    uint8_t secondDigit = (num / 100) % 10; // Získá druhou číslici (stovky)
    uint8_t thirdDigit  = (num / 10) % 10; // Získá třetí číslici (desítky)
    uint8_t fourthDigit = num % 10; // Získá čtvrtou číslici (jednotky)

    displaySymbol(HEX_ARRAY[firstDigit],AnclkDisplay::DisplayEnum::DISP1);
    displaySymbol(HEX_ARRAY[secondDigit],AnclkDisplay::DisplayEnum::DISP2);  
    displaySymbol(HEX_ARRAY[thirdDigit],AnclkDisplay::DisplayEnum::DISP3);
    displaySymbol(HEX_ARRAY[fourthDigit],AnclkDisplay::DisplayEnum::DISP4); 
    displayColon(false);  
}

void AnclkDisplay::showHexNumber(uint16_t num) {
    // Pokud je číslo větší než 0xFFFF (65535), ignorujeme jej
    if (num > 0xFFFF) return;

    // Výpočet jednotlivých znaků na pozicích
    uint8_t firstDigit  = (num >> 12) & 0xF; // První 4 bity (nejvyšší, pro první pozici)
    uint8_t secondDigit = (num >> 8) & 0xF;  // Dalších 4 bity (pro druhou pozici)
    uint8_t thirdDigit  = (num >> 4) & 0xF;  // Dalších 4 bity (pro třetí pozici)
    uint8_t fourthDigit = num & 0xF;         // Poslední 4 bity (nejnižší, pro čtvrtou pozici)

    // Zobrazení jednotlivých znaků
    displaySymbol(HEX_ARRAY[firstDigit], AnclkDisplay::DisplayEnum::DISP1);
    displaySymbol(HEX_ARRAY[secondDigit], AnclkDisplay::DisplayEnum::DISP2);  
    displaySymbol(HEX_ARRAY[thirdDigit], AnclkDisplay::DisplayEnum::DISP3);
    displaySymbol(HEX_ARRAY[fourthDigit], AnclkDisplay::DisplayEnum::DISP4); 
    displayColon(false);  
}

void AnclkDisplay::activateOutput(PowerOutput output, bool state) {
  switch (output) {
    case PowerOutput::OUT1: 
      bitWrite(expanderIC4Port, ANCLK_OUT1_12VDC, state);
      expanderIC4->digitalWrite(expanderIC4Port);  
      break;

    case PowerOutput::OUT2: 
      bitWrite(expanderIC4Port, ANCLK_OUT2_12VDC, state);
      expanderIC4->digitalWrite(expanderIC4Port);  
      break;

    case PowerOutput::OUT3: 
      bitWrite(expanderIC4Port, ANCLK_OUT3_12VDC, state);
      expanderIC4->digitalWrite(expanderIC4Port);  
      break;
  } 
}
