/**
 *  @file       AnclkDisplay.h
 *  Project     Anclk display library
 *  @brief      An Arduino library for custom made JSC electronics ANCLK display.
 *  @author     Robert Faltus for JSC electronics s.r.o.
 *  License     Apache-2.0 - Copyright (c) 2022 – 2023 JSC electronics
 *  @version    v1.0
 *
 *  Copyright (c) 2022 – 2023 JSC electronics
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef ANCLK_H
#define ANCLK_H

#include <Arduino.h>
#include <I2cDiscreteIoExpander.h> // by Doc Walker, https://github.com/4-20ma/I2cDiscreteIoExpander
//#include <PCF8574.h>
#include <Wire.h>
#include "AnclkPinout.h"

#define NUM_OF_INDEXES          4

#define INIT_VAL_PCF8575        0x0000
#define INIT_VAL_PCF8574        0x00

// letters, numbers and symbols
#define BLANK       ((byte)0b00000000)
#define DASH        ((byte)0b00000010)    

#define NUM_0       ((byte)0b11111100)
#define NUM_1       ((byte)0b01100000)
#define NUM_2       ((byte)0b11011010)
#define NUM_3       ((byte)0b11110010)
#define NUM_4       ((byte)0b01100110)
#define NUM_5       ((byte)0b10110110)
#define NUM_6       ((byte)0b10111110)
#define NUM_7       ((byte)0b11100000)
#define NUM_8       ((byte)0b11111110)
#define NUM_9       ((byte)0b11110110)
#define LETTER_A    ((byte)0b11101110)
#define LETTER_b    ((byte)0b00111110)
#define LETTER_C    ((byte)0b10011100) 
#define LETTER_d    ((byte)0b01111010) 
#define LETTER_E    ((byte)0b10011110) 
#define LETTER_F    ((byte)0b10001110) 

static constexpr byte HEX_ARRAY[16] = {
    NUM_0, NUM_1, NUM_2, NUM_3, NUM_4, NUM_5, NUM_6, NUM_7, NUM_8, NUM_9,
    LETTER_A, LETTER_b, LETTER_C, LETTER_d, LETTER_E, LETTER_F
};

#define LETTER_I    ((byte)0b01100000)
#define LETTER_H    ((byte)0b01101110)
#define LETTER_o    ((byte)0b00111010)
#define LETTER_r    ((byte)0b00001010)

// Define segment arrays for each display
static constexpr int segmentArray1[8] = {DISP1_A_PIN, DISP1_B_PIN, DISP1_C_PIN, DISP1_D_PIN, DISP1_E_PIN, DISP1_F_PIN, DISP1_G_PIN, DISP1_DP_PIN};
static constexpr int segmentArray2[8] = {DISP2_A_PIN, DISP2_B_PIN, DISP2_C_PIN, DISP2_D_PIN, DISP2_E_PIN, DISP2_F_PIN, DISP2_G_PIN, DISP2_DP_PIN};
static constexpr int segmentArray3[8] = {DISP3_A_PIN, DISP3_B_PIN, DISP3_C_PIN, DISP3_D_PIN, DISP3_E_PIN, DISP3_F_PIN, DISP3_G_PIN, DISP3_DP_PIN};
static constexpr int segmentArray4[8] = {DISP4_A_PIN, DISP4_B_PIN, DISP4_C_PIN, DISP4_D_PIN, DISP4_E_PIN, DISP4_F_PIN, DISP4_G_PIN, DISP4_DP_PIN};

class AnclkDisplay
{
private:
    I2cDiscreteIoExpander *expanderIC7;   //IC7
    I2cDiscreteIoExpander *expanderIC4;   //IC4    
    I2cDiscreteIoExpander *expanderIC2;   //IC2

    uint8_t cmdState = 0;

    uint16_t expanderIC7Port    = INIT_VAL_PCF8575; //16 bit expander initial value
    uint16_t expanderIC4Port    = INIT_VAL_PCF8575; //16 bit expander initial value
    uint16_t expanderIC2Port    = INIT_VAL_PCF8575; //16 bit expander initial value

public:
    enum class DisplayEnum {
        DISP1,
        DISP2,
        DISP3,
        DISP4
    };

    enum class PowerOutput {
        OUT1,
        OUT2,
        OUT3
    };

public:
    AnclkDisplay(uint8_t expAdr1, uint8_t expAdr2, uint8_t expAdr3);
    
    void begin();

    void displaySymbol(byte symbol, DisplayEnum disp);
    void displayColon(bool state = true);
    void displayDot(uint8_t idx, bool state = true);

    void displayClear();

    void showHexNumber(uint16_t num);
    void showNumber(uint16_t num);

    void printPortsState();
    void printPortsState(uint16_t expanderIC4Port, uint16_t expanderIC2Port, uint16_t expanderIC7Port);
    uint8_t readAllExpanders(uint16_t &exp1, uint16_t &exp2, uint16_t &exp3);

    void activateOutput(PowerOutput output, bool state);

private:
    bool isCorrectIdx(uint8_t idx);

    void printSingleDisplay(uint16_t port, int a, int b, int c, int d, int e, int f, int g, int dp);
};

#endif

